
/*
 * This is letting the user change the package selection using a dialog
 * based GUI.
 */

#include "gasgui.h"
#include <dialog.h>
#include <limits.h>
#include <sys/ioctl.h>

int screensize_x = 70;
int screensize_y = 19;

#define HELP_TEXT \
"Select and de-select packages.\n"					\
" [*] package installed       [+] package marked for installation\n"	\
" [ ] package not installed   [-] package marked for removal\n"		\
" [>] package marked for replacing currently installed version"

char ** items;
struct directory_entry ** entries;
int item_counter;

char *extra_dir_items[] = {
	"a", "Open entire directory tree      ",
	"b", "Close entire directory tree     ",
	"c", "Un-Install all packages in tree ",
	"d", "Reset all packages in tree      ",
	"e", "Install all packages in tree    ",
	"f", "Upgrade all packages in tree    ",
	"g", "Re-Install all packages in tree ",
	"h", "Show current action-list        "
};

char *extra_pkg_items[] = {
	"a", "Show package information        ",
	"b", "Show current action-list        "
};

void create_dialog_itemlist(struct directory * dir, int blanks)
{
        struct directory_entry * e;
	char tmpstring[512];
                        
        for (e = dir->list; e != NULL; e = e->next) {
		items[item_counter*2] = malloc(5);
		snprintf(items[item_counter*2], 5, "%4d", item_counter+1);
		entries[item_counter] = e;
		
                if ( e->content_is_subdir ) {
                        snprintf(tmpstring, 512,
			        "%*s(%c) %s/", blanks, "",
				e->content.dir->is_open_in_gui ? '-' : '+',
				e->content.dir->name);
                } else {
			char ch = e->content.pkg->is_installed ? '*' : ' ';
			char *remark = "";

			if ( e->content.pkg->action < 0 )
				ch = e->content.pkg->is_installed ? '-' : 'X';
			if ( e->content.pkg->action > 0 )
				ch = e->content.pkg->is_installed ? '>' : '+';

			if ( e->content.pkg->installed_version &&
			     strcmp(e->content.pkg->installed_version,
			            e->content.pkg->version) ) remark="(new)";

                        snprintf(tmpstring, 512, "%*s[%c] %s-%s %s",
				blanks, "", ch, e->content.pkg->name,
                                e->content.pkg->version, remark);
                }
		items[item_counter*2+1] = malloc(60);
		snprintf(items[item_counter*2+1], 60, "%-*s", 50, tmpstring);
		item_counter++;

		if ( e->content_is_subdir && e->content.dir->is_open_in_gui )
                        create_dialog_itemlist(e->content.dir, blanks+3);
        }
}

void set_tree_flag(struct directory * dir, int set_dir, int value)
{
        struct directory_entry * e;
                        
	if ( set_dir ) dir->is_open_in_gui = value;

        for (e = dir->list; e != NULL; e = e->next) {
                if ( e->content_is_subdir && set_dir)
			e->content.dir->is_open_in_gui = value;

		if ( !e->content_is_subdir && !set_dir) {
			if ( value == 0 ) {
				e->content.pkg->action = value;
			} else if ( value < 0 ) {
				e->content.pkg->action =
					e->content.pkg->is_installed ? -1 : 0;
			} else if ( value == 1 ) {
				if ( ! e->content.pkg->is_installed )
						e->content.pkg->action = 1;
			} else if ( value == 2 ) {
				if ( e->content.pkg->installed_version &&
				     strcmp(e->content.pkg->installed_version,
				            e->content.pkg->version) )
						e->content.pkg->action = 1;
			} else if ( value == 3 ) {
				if ( e->content.pkg->is_installed )
						e->content.pkg->action = 1;
			}
		}

		if ( e->content_is_subdir )
                        set_tree_flag(e->content.dir, set_dir, value);
        }
}

void free_dialog_item_list()
{
	for (item_counter--; item_counter>0; item_counter--) {
		free(items[item_counter*2+1]);
		free(items[item_counter*2]);
	}
}

void show_actions()
{
	struct package *p;
	char filename[200];
	FILE *f;
	int i;

	f = tmpfile();
	snprintf(filename, 200, "/proc/self/fd/%d", fileno(f));

	for (i=0, p=packages; p != NULL; p=p->next) if (p->action < 0) i++;

	if ( i ) {
		fprintf(f, "%d packages marked for removal:\n", i);

		for (i=0, p=packages; p != NULL; p=p->next)
			if ( p->action < 0 ) {
				fprintf(f, "%s-%s\n", p->name,
					p->installed_version ?
						p->installed_version : "?");
			}
	} else {
		fprintf(f, "No packages marked for removal.\n");
	}

	for (i=0, p=packages; p != NULL; p=p->next) if (p->action > 0) i++;

	if ( i ) {
		fprintf(f, "\n%d packages marked for installation:\n", i);

		for (i=0, p=packages; p != NULL; p=p->next)
			if ( p->action > 0 )
				fprintf(f, "%s-%s\n", p->name,
					p->installed_version ?
						p->installed_version : "?");
	} else {
		fprintf(f, "\nNo packages marked for installation.\n");
	}

	fflush(f); /* close afterwards because fclose() will remove file */
	dialog_textbox(" Current Action List ", filename,
	               screensize_y, screensize_x);
	fclose(f);
}

void show_pkginfo(struct package * p)
{
	char filename[PATH_MAX];
	char title[100];

	snprintf(filename, PATH_MAX, "%s/%s/info/packages/%s",
	         sourcedir, config, p->name);

	snprintf(title, 100, " Package Info for %s ", p->name);
	dialog_textbox(title, filename, screensize_y, screensize_x);
}

void extra_dir(struct directory * dir)
{
	int rc;

	dialog_vars.extra_button = 0;
	dialog_vars.cancel_label = "Cancel";

	rc = dialog_menu("Directory Extra Functions", "",
			14, 50, 8, 8, extra_dir_items);

	switch(dialog_vars.input_result[0]) {
		case 'a': set_tree_flag(dir, 1, 1); break;
		case 'b': set_tree_flag(dir, 1, 0); break;
		case 'c': set_tree_flag(dir, 0,-1); break;
		case 'd': set_tree_flag(dir, 0, 0); break;
		case 'e': set_tree_flag(dir, 0, 1); break;
		case 'f': set_tree_flag(dir, 0, 2); break;
		case 'g': set_tree_flag(dir, 0, 3); break;
		case 'h': show_actions(); break;
	}

	dialog_vars.extra_button = 1;
	dialog_vars.cancel_label = "Ok";
}

void extra_pkg(struct package * pkg)
{
	int rc;

	dialog_vars.extra_button = 0;
	dialog_vars.cancel_label = "Cancel";

	rc = dialog_menu("Package Extra Functions", "",
			8, 50, 2, 2, extra_pkg_items);

	switch(dialog_vars.input_result[0]) {
		case 'a': show_pkginfo(pkg); break;
		case 'b': show_actions();    break;
	}

	dialog_vars.extra_button = 1;
	dialog_vars.cancel_label = "Ok";
}

int do_selection()
{
	struct winsize ws;
	int rc;

	/* Create enought space for the items[] and entries[] arrays */
	items = malloc( (directory_entry_count+20) * sizeof(char*) * 2 );
	entries = malloc( (directory_entry_count+20) * sizeof(void*) );

	dialog_vars.default_item = malloc(10);
	dialog_vars.cancel_label = "Ok";
	dialog_vars.ok_label     = "Select";
	dialog_vars.extra_label  = "Extras";
	dialog_vars.extra_button = 1;
	strcpy(dialog_vars.default_item, "1");

	if ( !ioctl(1, TIOCGWINSZ, &ws) ) {
		screensize_x = ws.ws_col - 10;
		screensize_y = ws.ws_row - 5;
	}

	init_dialog();
	while (1) {
		put_backtitle();

		item_counter=0;
		create_dialog_itemlist(rootdir, 0);

		rc = dialog_menu(" G.A.S. (GEM Autoinstall Shell) GUI ",
				HELP_TEXT, screensize_y, screensize_x,
				screensize_y-10, item_counter, items);
		if ( rc == 3 ) {
			rc = atoi(dialog_vars.input_result) - 1;
			strcpy(dialog_vars.default_item,
			       dialog_vars.input_result);
			free_dialog_item_list();

			if (entries[rc]->content_is_subdir)
				extra_dir(entries[rc]->content.dir);
			else
				extra_pkg(entries[rc]->content.pkg);

			continue;
		}
		if ( rc ) break;

		rc = atoi(dialog_vars.input_result) - 1;

		if (entries[rc]->content_is_subdir) {
			entries[rc]->content.dir->is_open_in_gui =
				! entries[rc]->content.dir->is_open_in_gui;
		} else {
			if ( entries[rc]->content.pkg->action == 0 )
				entries[rc]->content.pkg->action = 1;
			else if ( entries[rc]->content.pkg->action < 0 )
				entries[rc]->content.pkg->action = 0;
			else if ( entries[rc]->content.pkg->action > 0 )
				entries[rc]->content.pkg->action =
					entries[rc]->content.pkg->
						is_installed ? -1 : 0;
		}

		strcpy(dialog_vars.default_item, dialog_vars.input_result);
		free_dialog_item_list();
	}
	free_dialog_item_list();
	end_dialog();

	free(items);
	free(entries);

	free(dialog_vars.default_item);
	dialog_vars.default_item = NULL;

	return 0;
}

